<?php

namespace App\Http\Middleware;

use App\Model\Depart;
use App\Service\EmployeeService;
use Closure;
use App\Service\TokenService;

/**
 * 用户登陆中间件
 */
class CheckLogin
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $token=$request->header('Authorization');
        if (!isset($token)){
            return  response()->json(['code'=>1,'msg'=>'缺少token','data'=>null]);
        }

        //校验token
        $result = TokenService::verifyToken($token);
        if ($result <= 0){
            return response()->json(['code'=>1,'msg'=>TokenService::error[$result],'data'=>null]);
        }

        //校验用户
        $checkResult = EmployeeService::checkUser($result);
        list($state, $data) = $checkResult;
        if(! $state) return response()->json(['code'=>1,'msg'=>$data,'data'=>null]);

        //人员角色
        $data['role'] = EmployeeService::getPersonRole($result);
        $return = EmployeeService::getLoginDepart($result);
        //所属部门
        $data['rule_depart'] = $return[0] ?? [];
        //顶级公司
        $data['depart_top'] = $return[1] ?? [];
        //部门对应的顶级公司
        $data['depart_map'] = $return[2] ?? [];
        //权限范围内的部门以及公司
        $data['depart_range'] = $return[3] ?? [];
        //是否有所有的部门权限
        $data['is_all_depart'] = $return[4] ?? 0;
        //总公司
        $data['head'] = $return[5] ?? [];
        //是否是总公司下的人
        $data['is_behind_main'] = $return[6] ?? 0;
        //是否库存校验
        $data['is_check_stock'] = true;

        //写入user信息
        $request->userData = $data;

        return $next($request);
    }
}
