<?php

namespace App\Service;

use App\Model\Depart;
use App\Model\Employee;
use App\Model\InvoiceOrder;
use App\Model\InvoiceOrderInfo;
use App\Model\SalesOrder;
use App\Model\SeeRange;
use App\Model\Storehouse;
use Illuminate\Support\Facades\DB;

/**
 * 发货单
 */
class InvoiceOrderService extends Service
{
    /**
     * 发货单编辑
     * @param $data
     * @param $user
     * @return array
     */
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,$user,false);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = InvoiceOrder::where('order_number',$data['order_number'])->first();
            $material_model->sales_order_id = $data['sales_order_id'];
            $material_model->send = $data['send'];
            $material_model->send_time = $data['send_time'];
            $material_model->take = $data['take'];
            $material_model->take_address = $data['take_address'];
            $material_model->take_phone = $data['take_phone'];
            $material_model->logistics_company = $data['logistics_company'];
            $material_model->logistics_number = $data['logistics_number'];
            $material_model->mark = $data['mark'];
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            InvoiceOrderInfo::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'invoice_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                InvoiceOrderInfo::insert($sub);

                //锁定库存
                ProductInventoryService::changeLockNumber($user, $msg[0],$msg[1]);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 发货单新增
     * @param $data
     * @param $user
     * @return array
     */
    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data, $user);
        if(!$status) return [$status,$msg];

        try{
            DB::beginTransaction();
            $material_model = new InvoiceOrder();
            $material_model->order_number = $data['order_number'];
            $material_model->sales_order_id = $data['sales_order_id'];
            $material_model->send = $data['send'];
            $material_model->send_time = $data['send_time'];
            $material_model->take = $data['take'];
            $material_model->take_address = $data['take_address'];
            $material_model->take_phone = $data['take_phone'];
            $material_model->logistics_company = $data['logistics_company'];
            $material_model->logistics_number = $data['logistics_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->mark = $data['mark'];
            $material_model->crt_id = $user['id'];
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'invoice_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'mark' => $value['mark'] ?? '',
                    ];
                }
                InvoiceOrderInfo::insert($sub);

                //锁定库存
                ProductInventoryService::changeLockNumber($user,$msg[0],[]);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 发货单详情
     * @param $data
     * @return array
     */
    public function detail($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据'];

        $order = InvoiceOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->select('id','order_number','sales_order_id','send','send_time','take','take_address','take_phone','logistics_company','logistics_number','crt_id','mark','state','crt_time','storehouse_id','top_depart_id','depart_id')
            ->first();
        if(empty($order)) return [false, '发货订单不存在或已被删除'];
        $order = $order->toArray();
        $order['storehouse_title'] = Storehouse::where('id',$order['storehouse_id'])->value('title');
        $order['state_title'] = InvoiceOrder::$name[$order['state']] ?? '';
        $sales = SalesOrder::where('id',$order['sales_order_id'])->value('order_number');
        $order['sales_order_number'] = $sales;

        $emp_map = Employee::whereIn('id',array_unique(array_merge_recursive([$order['crt_id']],[$order['send']])))
            ->pluck('emp_name','id')
            ->toArray();
        $info = InvoiceOrderInfo::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->get()->toArray();
        $map = (new ProductService())->getProductDetail(array_column($info,'product_id'));
        foreach ($info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            $value['title'] = $tmp['title'] ?? "";
            $value['code'] = $tmp['code'] ?? "";
            $value['size'] = $tmp['size'] ?? "";
            $value['unit'] = $tmp['unit'] ?? "";
            $value['bar_code'] = $tmp['bar_code'] ?? "";
            $order['product'][] = $value;
        }
        $order['send_name'] = $emp_map[$order['send']] ?? '';
        $order['crt_name'] = $emp_map[$order['crt_id']] ?? '';
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';
        //可见范围
        $return = (new RangeService())->RangeDetail($order['id'],SeeRange::type_three);
        $order['depart'] = $return[0] ?? [];
        $order['employee'] = $return[1] ?? [];

        return [true, $order];
    }

    /**
     * 发货单删除
     * @param $data
     * @return array
     */
    public function del($data, $user){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据！'];

        $order = InvoiceOrder::where('order_number',$data['order_number'])
            ->first();
        if(empty($order)) return [false,'发货单不存在或已被删除'];
        $order = $order->toArray();
        if($order['state'] > InvoiceOrder::STATE_ZERO) return [false,'发货订单已确认,删除失败！'];

        $product_save = $this->getSaveDetail($data['order_number']);
        try{
            DB::beginTransaction();
            InvoiceOrder::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);
            InvoiceOrderInfo::where('order_number',$data['order_number'])->update([
                'del_time'=>time()
            ]);
            (new RangeService())->RangeDelete($order['id'],SeeRange::type_three);
            //锁定库存释放
            ProductInventoryService::changeLockNumber($user,[],$product_save);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 获取发货单列表
     * @param $data
     * @param $user
     * @return array
     */
    public function getList($data,$user){
        $model = InvoiceOrder::Clear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','order_number','sales_order_id','depart_id','send','send_time','take','take_address','take_phone','logistics_company','logistics_number','crt_id','mark','state','crt_time','storehouse_id')
            ->orderby('id', 'desc');
        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }
        if(! empty($data['time_type'])) {
            if($data['time_type'] == 1) {
                $start = strtotime('today');
                $end = strtotime('tomorrow') - 1;
            }elseif ($data['time_type'] == 2){
                $start = strtotime('this week',strtotime('today'));
                $end = strtotime('this week +6 days 23:59:59', strtotime('today'));
            }
            if(! empty($start) && ! empty($end)) {
                $model->where('crt_time','>=',$start);
                $model->where('crt_time','<=',$end);
            }
        }
        if(! empty($data['send_man'])){
            $emp = Employee::where('del_time',0)
                ->where('emp_name', 'LIKE', '%'.$data['send_man'].'%')
                ->select('id')
                ->get()->toArray();
            $model->whereIn('send',array_unique(array_column($emp,'id')));
        }
        if(! empty($data['take'])) $model->where('take', 'LIKE', '%'.$data['take'].'%');
        if(! empty($data['sale_order'])){
            $model2 = SalesOrder::Clear($user,$data);
            $sale = $model2->where('del_time',0)
                ->where('order_number', 'LIKE', '%'.$data['sale_order'].'%')
                ->select('id')
                ->get()->toArray();
            $model->whereIn('sales_order_id',array_unique(array_column($sale,'id')));
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list);

        return [true, $list];
    }

    /**
     * 拼接数据
     * @param $data
     * @return array
     */
    public function fillListData($data){
        if(empty($data['data'])) return $data;

        $depart_id = array_unique(array_column($data['data'],'depart_id'));
        $depart_map = Depart::whereIn('id',$depart_id)
            ->pluck('title','id')
            ->toArray();

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id'),array_column($data['data'],'send')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();

        $sales = SalesOrder::whereIn('id',array_unique(array_column($data['data'],'sales_order_id')))->pluck('order_number','id')->toArray();
        $storehouse = Storehouse::whereIn('id',array_unique(array_column($data['data'],'storehouse_id')))
            ->pluck('title','id')
            ->toArray();
        foreach ($data['data'] as $key => $value){
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['depart_name'] = $depart_map[$value['depart_id']] ?? '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['send_name'] = $emp_map[$value['send']] ?? '';
            $data['data'][$key]['state_title'] = InvoiceOrder::$name[$value['state']] ?? '';
            $data['data'][$key]['sales_order_number'] = $sales[$value['sales_order_id']] ?? '';
            $data['data'][$key]['storehouse_title'] = $storehouse[$value['storehouse_id']] ?? '';
        }

        return $data;
    }

    /**
     * 发货单参数规则
     * @param $data
     * @param $is_check
     * @return array
     */
    public function orderRule(&$data, $user, $is_check = true){
        if($this->isEmpty($data,'send')) return [false,'发货人不能为空！'];
        if($this->isEmpty($data,'product')) return [false,'产品不能为空！'];
        if(empty($data['storehouse_id'])) return [false,'请选择仓库'];
        if(! empty($data['send_time'])) $data['send_time'] = $this->changeDateToDateMin($data['send_time']);
        if(! empty($data['sales_order_id'])) {
            $sales = SalesOrder::where('del_time',0)
                ->where('id',$data['sales_order_id'])
                ->first();
            if(empty($sales)) return [false,'合同不存在或已被删除'];
//            if($sales['sales_order_type'] != SalesOrder::Order_type_two) return [false,'非快递件合同'];
        }

        //所属部门 以及  顶级部门
        if(empty($data['depart_id'])) {
            $data['depart_id'] = $this->getDepart($user);
            $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;
        }

        $product_submit = $product_id = [];
        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品量'];

            $key = $value['product_id'] . ',' .$data['storehouse_id'];
            if(isset($product_submit[$value['product_id']])){
                $product_submit[$key] += $value['number'];
            }else{
                $product_submit[$key] = $value['number'];
            }
            $product_id[] = $value['product_id'];
        }
        $order_number = $data['order_number'] ?? '';
        $product_save = $this->getSaveDetail($order_number);

        //是否校验库存
        ProductInventoryService::is_check($user,$data);

        list($status,$msg) = (new ProductInventoryService())->compareStock($user,$product_id, $product_submit, $product_save);
        if(! $status) return [false, $msg];

        if($is_check){
            $order_number = (new OrderNoService())->createOrderNumber(InvoiceOrder::prefix);
            if(empty($order_number)) return [false,'发货单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'id')) return [false,'发货单ID不能为空！'];
            if($this->isEmpty($data,'order_number')) return [false,'发货单号不能为空！'];
        }

        return [true, [$product_submit, $product_save]];
    }

    /**
     * 获取保存详情
     * @param $order_number
     * @return array
     */
    public function getSaveDetail($order_number){
        $product_save = [];
        if(empty($order_number)) return $product_save;

        $sub = InvoiceOrderInfo::where('order_number',$order_number)
            ->where('del_time',0)
            ->get()->toArray();
        foreach ($sub as $value){
            $key = $value['product_id'] . ',' . $value['storehouse_id'];
            if(isset($product_save[$key])){
                $product_save[$key] += $value['number'];
            }else{
                $product_save[$key] = $value['number'];
            }
        }

        return $product_save;
    }
}
