<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\BookingList;
use App\Model\Depart;
use App\Model\Employee;
use App\Model\PurchaseOrder;
use App\Model\PurchaseOrderFile;
use App\Model\PurchaseOrderInfo;
use App\Model\ReturnExchangeOrder;
use App\Model\SalesOrder;
use App\Model\SeeRange;
use App\Model\Storehouse;
use App\Model\Supplier;
use Illuminate\Support\Facades\DB;

/**
 * 采购单
 */
class PurchaseOrderService extends Service
{
    /**
     * 采购单编辑
     * @param $data
     * @param $user
     * @return array
     */
    public function edit($data,$user){
        list($status,$msg) = $this->orderRule($data,$user, false);
        if(!$status) return [$status,$msg];

        $params = $this->getDataFile($data);
        (new OperationLogService())->setOperationList($params,$user,2);

        try{
            DB::beginTransaction();
            $material_model = PurchaseOrder::where('order_number',$data['order_number'])->first();
            $material_model->purchase_type = $data['purchase_type'] ?? 0;
            $material_model->order_number = $data['order_number'];
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? 0;
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            $time = time();
            PurchaseOrderInfo::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                        'rate' => $value['rate'] ?? 0,
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            PurchaseOrderFile::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'file' => $value['url'],
                        'type' => PurchaseOrderFile::type_one,
                        'name' => $value['name'],
                        'crt_time' => $time,
                    ];
                }
                PurchaseOrderFile::insert($insert);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 采购单新增
     * @param $data
     * @param $user
     * @return array
     */
    public function add($data,$user){
        list($status,$msg) = $this->orderRule($data, $user);
        if(!$status) return [$status,$msg];
        try{
            DB::beginTransaction();
            $material_model = new PurchaseOrder();
            $material_model->order_number = $data['order_number'];
            $material_model->purchase_type = $data['purchase_type'] ?? 0;
            $material_model->depart_id = $data['depart_id'] ?? 0;
            $material_model->top_depart_id = $data['top_depart_id'] ?? 0;
            $material_model->total = $data['total'] ?? 0;
            $material_model->other_fee = $data['other_fee'] ?? 0;
            $material_model->discount_fee = $data['discount_fee'] ?? 0;
            $material_model->purchase_total = $data['purchase_total'] ?? 0;
            $material_model->supplier = $data['supplier'] ?? 0;
            $material_model->purchase_id = $data['purchase_id'] ?? 0;
            $material_model->order_type = $data['order_type'];
            $material_model->sales_order_id = $data['sales_order_id'] ?? 0;
            $material_model->mark = $data['mark'] ?? '';
            $material_model->crt_id = $user['id'];
            $material_model->storehouse_id = $data['storehouse_id'];
            $material_model->save();

            $time = time();
            if(!empty($data['product'])){
                $sub = [];
                foreach ($data['product'] as $value){
                    $sub[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'storehouse_id' => $data['storehouse_id'] ?? 0,
                        'product_id' => $value['product_id'],
                        'number' => $value['number'],
                        'basic_type_id' => $value['basic_type_id'],
                        'price' => $value['price'],
                        'mark' => $value['mark'] ?? '',
                        'rate' => $value['rate'] ?? 0,
                    ];
                }
                PurchaseOrderInfo::insert($sub);
            }

            if(! empty($data['file'])){
                $insert = [];
                foreach ($data['file'] as $value){
                    $insert[] = [
                        'purchase_order_id' => $material_model->id,
                        'order_number' => $data['order_number'],
                        'file' => $value['url'],
                        'type' => PurchaseOrderFile::type_one,
                        'name' => $value['name'],
                        'crt_time' => $time,
                    ];
                }
                PurchaseOrderFile::insert($insert);
            }

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        (new OperationLogService())->setOperationList($data,$user);

        return [true,''];
    }

    /**
     * 采购单详情
     * @param $data
     * @return array
     */
    public function detail($data,$user){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->select('id','order_number','depart_id','mark','state','crt_id','order_type','sales_order_id','supplier','purchase_id','total','other_fee','discount_fee','purchase_total','crt_time','storehouse_id','top_depart_id','purchase_type')
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        $order = $order->toArray();
        $order['file'] = [];
        $order['purchase_type_title'] = BasicType::where('id',$order['purchase_type'])->value('title') ?? '';
        $order['storehouse_title'] = Storehouse::where('id',$order['storehouse_id'])->value('title');
        $supplier = Supplier::where('id',$order['supplier'])->first();
        $order['supplier_title'] = $supplier->title ?? '';
        $order['order_type_title'] = PurchaseOrder::$order_type[$order['order_type']] ?? '';
        $order['state_title'] = PurchaseOrder::$name[$order['state']] ?? '';
        $sales = SalesOrder::where('id',$order['sales_order_id'])->value('order_number');
        $order['sales_order_number'] = $sales;
        $emp_map = Employee::whereIn('id',array_unique(array_merge_recursive([$order['purchase_id']],[$order['crt_id']])))
            ->pluck('emp_name','id')
            ->toArray();
        $order['purchase_title'] = $emp_map[$order['purchase_id']] ?? '';

        $p_info = PurchaseOrderInfo::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->get()->toArray();
        $basic_price = BasicType::whereIn('id',array_unique(array_column($p_info,'basic_type_id')))->pluck('title','id')->toArray();
        $map = (new ProductService())->getProductDetail(array_column($p_info,'product_id'));
        foreach ($p_info as $value){
            $tmp = $map[$value['product_id']] ?? [];
            $value['title'] = $tmp['title'] ?? "";
            $value['code'] = $tmp['code'] ?? "";
            $value['size'] = $tmp['size'] ?? "";
            $value['unit'] = $tmp['unit'] ?? "";
            $value['bar_code'] = $tmp['bar_code'] ?? "";
            $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
            $order['product'][] = $value;
        }

        $file = PurchaseOrderFile::where('del_time',0)
            ->where('order_number',$data['order_number'])
            ->get()->toArray();
        foreach ($file as $value){
            if($value['type'] == PurchaseOrderFile::type_one){
                $tmp = [
                    'url' => $value['file'],
                    'name' => $value['name'],
                ];
                $order['file'][] = $tmp;
            }
        }
        $order['crt_name'] = $emp_map[$order['crt_id']] ?? '';
        $order['crt_time'] = $order['crt_time'] ? date("Y-m-d H:i:s",$order['crt_time']): '';
        //可见范围
        $return = (new RangeService())->RangeDetail($order['id'],SeeRange::type_five);
        $order['depart'] = $return[0] ?? [];
        $order['employee'] = $return[1] ?? [];

        //退换货信息
        $order['return_exchange'] = (new ReturnExchangeOrderService())->getDifferentAmount($order['id'],ReturnExchangeOrder::Order_type2);

        //总社id
        $top_depart_id = $user['head'] ?? [];
        $top_depart_id = $top_depart_id['id'] ?? 0;
        if(empty($order['state'])) {
            //分社的采购单 供应商是总社
            if($order['top_depart_id'] != $top_depart_id && ! empty($supplier->is_main)) {
                $is_show_confirm = 0;
                //总公司下的人
                if($user['is_behind_main']) $is_show_confirm = 1;
            }else{
                $is_show_confirm = 1;
            }
        }else{
            $is_show_confirm = 0;
        }
        $order['is_show_confirm'] = $is_show_confirm;

        return [true, $order];
    }

    /**
     * 采购单删除
     * @param $data
     * @return array
     */
    public function del($data){
        if($this->isEmpty($data,'order_number')) return [false,'请选择数据！'];
        $order = PurchaseOrder::where('del_time',0)->where('order_number',$data['order_number'])->first();
        if(empty($order)) return [false,'采购单不存在或已被删除'];
        $order = $order->toArray();
        if($order['state'] > PurchaseOrder::STATE_ZERO) return [false,'请确认采购订单状态,删除失败！'];

        try{
            DB::beginTransaction();
            PurchaseOrder::where('order_number',$data['order_number'])->where('del_time',0)->update([
                'del_time'=>time()
            ]);
            PurchaseOrderInfo::where('order_number',$data['order_number'])->where('del_time',0)->update([
                'del_time'=>time()
            ]);
            PurchaseOrderFile::where('order_number',$data['order_number'])
                ->where('del_time',0)
                ->update(['del_time' => time()]);
            (new RangeService())->RangeDelete($order['id'],SeeRange::type_five);

            DB::commit();
        }catch (\Throwable $e){
            DB::rollBack();
            return [false,$e->getMessage()];
        }

        return [true,''];
    }

    /**
     * 获取采购单列表
     * @param $data
     * @param $user
     * @return array
     */
    public function getList($data,$user){
        $model = PurchaseOrder::Clear($user,$data);
        $model = $model->where('del_time',0)->select('order_number','id','depart_id','top_depart_id','mark','crt_time','state','crt_id','order_type','sales_order_id','supplier','purchase_id','purchase_total','discount_fee','other_fee','total','storehouse_id')
            ->orderby('id', 'desc');
        if(! empty($data['order_number'])) $model->where('order_number', 'LIKE', '%'.$data['order_number'].'%');
        if(! empty($data['depart_id'])) $model->where('depart_id', $data['depart_id']);
        if(! empty($data['company_id'])) $model->where('company_id', $data['company_id']);
        if(isset($data['state'])) $model->where('state',$data['state']);
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->whereBetween('crt_time',[$return[0],$return[1]]);
        }
        if(! empty($data['time_type'])) {
            if($data['time_type'] == 1) {
                $start = strtotime('today');
                $end = strtotime('tomorrow') - 1;
            }elseif ($data['time_type'] == 2){
                $start = strtotime('this week',strtotime('today'));
                $end = strtotime('this week +6 days 23:59:59', strtotime('today'));
            }
            if(! empty($start) && ! empty($end)) {
                $model->where('crt_time','>=',$start);
                $model->where('crt_time','<=',$end);
            }
        }
        if(! empty($data['order_type'])) {
            if($data['order_type'] == PurchaseOrder::Order_type_five){
                $model->whereIn('order_type',[PurchaseOrder::Order_type_two,PurchaseOrder::Order_type_three,PurchaseOrder::Order_type_four]);
            }else{
                $model->where('order_type',$data['order_type']);
            }
        }
        if(! empty($data['sale_order'])){
            $model2 = SalesOrder::Clear($user,$data);
            $sale = $model2->where('del_time',0)
                ->where('order_number', 'LIKE', '%'.$data['sale_order'].'%')
                ->select('id')
                ->get()->toArray();
            $model->whereIn('sales_order_id',array_unique(array_column($sale,'id')));
        }
        if(! empty($data['supplier_man'])){
            $emp = Supplier::where('del_time',0)
                ->where('title', 'LIKE', '%'.$data['supplier_man'].'%')
                ->select('id')
                ->get()->toArray();
            $model->whereIn('supplier',array_unique(array_column($emp,'id')));
        }
        if(! empty($data['purchase_man'])){
            $emp = Employee::where('del_time',0)
                ->where('emp_name', 'LIKE', '%'.$data['purchase_man'].'%')
                ->select('id')
                ->get()->toArray();
            $model->whereIn('purchase_id',array_unique(array_column($emp,'id')));
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillListData($list,$data,$user);

        return [true, $list];
    }

    /**
     * 拼接数据
     * @param $data
     * @return array
     */
    public function fillListData($data,$ergs,$user){
        if(empty($data['data'])) return $data;

        $depart_id = array_unique(array_column($data['data'],'depart_id'));
        $depart_map = Depart::whereIn('id',$depart_id)
            ->pluck('title','id')
            ->toArray();

        $emp_id = array_unique(array_merge_recursive(array_column($data['data'],'crt_id'),array_column($data['data'],'purchase_id')));
        $emp_map = Employee::whereIn('id',$emp_id)
                    ->pluck('emp_name','id')
                    ->toArray();

        $sales = SalesOrder::whereIn('id',array_unique(array_column($data['data'],'sales_order_id')))->pluck('order_number','id')->toArray();
        $storehouse = Storehouse::whereIn('id',array_unique(array_column($data['data'],'storehouse_id')))
            ->pluck('title','id')
            ->toArray();
        $supplier = Supplier::whereIn('id',array_unique(array_column($data['data'],'supplier')))
            ->select('title','id','is_main')
            ->get()
            ->toArray();
        $supplier_map = array_column($supplier,null,'id');
        $product = [];
        if(! empty($ergs['product'])){
            $sales_p_info = PurchaseOrderInfo::where('del_time',0)
                ->whereIn('purchase_order_id',array_column($data['data'],'id'))
                ->get()->toArray();
            $basic_price = BasicType::whereIn('id',array_unique(array_column($sales_p_info,'basic_type_id')))->pluck('title','id')->toArray();
            $map = (new ProductService())->getProductDetail(array_column($sales_p_info,'product_id'));

            foreach ($sales_p_info as $value){
                $tmp = $map[$value['product_id']] ?? [];
                $value['title'] = $tmp['title'] ?? "";
                $value['code'] = $tmp['code'] ?? "";
                $value['size'] = $tmp['size'] ?? "";
                $value['unit'] = $tmp['unit'] ?? "";
                $value['bar_code'] = $tmp['bar_code'] ?? "";
                $value['basic_type_title'] = $basic_price[$value['basic_type_id']] ?? "";
                $product[$value['purchase_order_id']][] = $value;
            }
        }
        //获取金额列表
        $money = (new BookingListService())->getAllAmount(array_column($data['data'],'id'), BookingList::data_type_two);
        foreach ($data['data'] as $key => $value){
            $supplier_tmp = $supplier_map[$value['supplier']] ?? [];
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date("Y-m-d H:i:s",$value['crt_time']) : '';
            $data['data'][$key]['depart_name'] = $depart_map[$value['depart_id']] ?? '';
            $data['data'][$key]['crt_name'] = $emp_map[$value['crt_id']] ?? '';
            $data['data'][$key]['purchase_name'] = $emp_map[$value['purchase_id']] ?? '';
            $data['data'][$key]['order_type_title'] = PurchaseOrder::$order_type[$value['order_type']] ?? '';
            $data['data'][$key]['state_title'] = PurchaseOrder::$name[$value['state']] ?? '';
            $data['data'][$key]['sales_order_number'] = $sales[$value['sales_order_id']] ?? '';
            $data['data'][$key]['storehouse_title'] = $storehouse[$value['storehouse_id']] ?? '';
            $data['data'][$key]['supplier_title'] = $supplier_tmp['title'] ?? '';
            $data['data'][$key]['product'] = $product[$value['id']] ?? [];

            //合同对应的金额数据
            $tmp = $money[$value['id']] ?? [];
            $fee1 = $tmp[BookingList::type_one] ?? 0; //收款金额
            $fee2 = $tmp[BookingList::type_two] ?? 0; //坏账金额
            $fee3 = $tmp[BookingList::type_three] ?? 0; //退款金额
            $fee4 = $tmp[BookingList::type_four] ?? 0; //补收金额

            //已收金额
            $data['data'][$key]['fee1'] = $fee1 + $fee4;
            //未收金额
            $data['data'][$key]['fee2'] = $value['purchase_total'] - ($fee1 + $fee4);
            //坏账金额
            $data['data'][$key]['fee3'] = $fee2;
            //最终金额
            $data['data'][$key]['fee4'] = $value['purchase_total'] + $fee4 - $fee3;
        }

        return $data;
    }

    /**
     * 采购单参数规则
     * @param $data
     * @param $is_check
     * @return array
     */
    public function orderRule(&$data, $user, $is_check = true){
        if(empty($data['order_type'])) return [false,'采购订单类型不能为空'];
        if($data['order_type'] == PurchaseOrder::Order_type_two && ! empty($data['supplier'])) {
            $supplier = Supplier::where('id',$data['supplier'])->first();
            if(! empty($supplier->is_main)) return [false,PurchaseOrder::$order_type[PurchaseOrder::Order_type_two] . '供应商请选择非总社供应商'];
        }
        if($data['order_type'] == PurchaseOrder::Order_type_three || $data['order_type'] == PurchaseOrder::Order_type_four) {
            if(empty($data['supplier'])){
                return [false,'请选择总部供应商'];
            }else{
                $supplier = Supplier::where('id',$data['supplier'])->first();
                if(empty($supplier->is_main)) return [false,'请选择总社供应商'];
            }
        }
        if($data['order_type'] == PurchaseOrder::Order_type_three && ! empty($data['sales_order_id'])) return [false,PurchaseOrder::$order_type[PurchaseOrder::Order_type_three] . '不需要选择合同'];
        if($data['order_type'] == PurchaseOrder::Order_type_four && empty($data['sales_order_id'])) return [false, PurchaseOrder::$order_type[PurchaseOrder::Order_type_four] . '必须选择合同'];
        if($this->isEmpty($data,'product')) return [false,'产品不能为空'];
        if(empty($data['storehouse_id'])) return [false,'请选择仓库'];
        if(! empty($data['other_fee'])){
            $res = $this->checkNumber($data['other_fee']);
            if(! $res) return [false, '其它费用请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['discount_fee'])){
            $res = $this->checkNumber($data['discount_fee']);
            if(! $res) return [false, '优惠金额请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['total'])){
            $res = $this->checkNumber($data['total']);
            if(! $res) return [false, '产品总价请输入不超过两位小数并且大于0的数值'];
        }
        if(! empty($data['purchase_total'])){
            $res = $this->checkNumber($data['purchase_total']);
            if(! $res) return [false, '采购总价请输入不超过两位小数并且大于0的数值'];
        }
        foreach ($data['product'] as $value){
            if(empty($value['number'])) return [false,'产品数量不能为空'];
            $res = $this->checkNumber($value['number']);
            if(! $res) return [false,'请输入正确的产品数量'];
        }

        //所属部门 以及 顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        if($is_check){
            $order_number = (new OrderNoService())->createOrderNumber(PurchaseOrder::prefix);
            if(empty($order_number)) return [false,'采购单号生成失败！'];
            $data['order_number'] = $order_number;
        }else{
            if($this->isEmpty($data,'order_number')) return [false,'采购单号不能为空！'];
        }

        return [true, ''];
    }

    /**
     * 采购单确认
     * @param $data
     * @param $user
     * @return array
     */
    public function purchaseOrderConfirm($data,$user){
        if(empty($data['order_number'])) return [false, '请选择数据'];

        $order = PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->first();
        if(empty($order)) return [false, '采购订单不存在或已被删除'];
        if($order->state != PurchaseOrder::STATE_ZERO) return [false, '请确认采购订单状态，操作失败'];

        PurchaseOrder::where('order_number',$data['order_number'])
            ->where('del_time',0)
            ->update(['state' => PurchaseOrder::STATE_ONE]);

        return [true,''];
    }
}
