<?php

namespace App\Service;

use App\Model\ConstructionInfo;
use App\Model\CustomerInfo;
use App\Model\Depart;
use App\Model\Employee;
use App\Model\Product;
use App\Model\SalesOrderInfo;
use App\Model\SeeRange;
use Illuminate\Support\Facades\DB;

class RangeService extends Service
{
    //设置可见范围 除了合同
    public function seeRange($data,$user){
        if(empty($data['data_type'])) return [false, "可见范围数据类型不能为空"];
        if(! in_array($data['data_type'], SeeRange::$type)) return [false, "可见范围数据类型错误"];
        if(empty($data['data_id'])) return [false,'可见范围数据ID不能为空'];

        $time = time();
        SeeRange::where('del_time',0)
            ->where('data_type',$data['data_type'])
            ->where('data_id',$data['data_id'])
            ->update(['del_time' => $time]);

        if(! empty($data['depart'])){
            $insert = [];
            foreach ($data['depart'] as $value){
                $insert[] = [
                    'data_id' => $data['data_id'],
                    'data_type' => $data['data_type'],
                    'param_id' => $value,
                    'type' => SeeRange::data_one,
                    'crt_time' => $time,
                ];
            }
            SeeRange::insert($insert);
        }

        if(! empty($data['employee'])){
            $insert = [];
            foreach ($data['employee'] as $value){
                $insert[] = [
                    'data_id' => $data['data_id'],
                    'data_type' => $data['data_type'],
                    'param_id' => $value,
                    'type' => SeeRange::data_two,
                    'crt_time' => $time,
                ];
            }
            SeeRange::insert($insert);
        }

        return [true,''];
    }

    //可见范围删除
    public function RangeDelete($data_id = 0, $data_type = 0){
        if(empty($data_id) || empty($data_type)) return;

        SeeRange::where('del_time',0)
            ->where('data_id',$data_id)
            ->where('data_type',$data_type)
            ->update(['del_time'=> time()]);
    }

    //获取可见范围详情
    public function RangeDetail($data_id = 0, $data_type = 0){
        if(empty($data_id) || empty($data_type)) return [];

        $see = SeeRange::where('del_time',0)
            ->where('data_id',$data_id)
            ->where('data_type',$data_type)
            ->get()->toArray();
        $depart_map = Depart::where('del_time',0)->pluck('title','id')->toArray();
        $emp_map = Employee::where('del_time',0)->pluck('emp_name','id')->toArray();

        $depart = $employee = $depart2 = [];
        foreach ($see as $value){
            if ($value['type'] == SeeRange::data_one){
                $name = $depart_map[$value['param_id']] ?? "";
                if(! empty($name)){
                    $tmp = [
                        'id' => $value['param_id'],
                        'name' => $depart_map[$value['param_id']] ?? "",
                    ];
                    $depart[] = $tmp;
                }
            }elseif ($value['type'] == SeeRange::data_two){
                $name = $emp_map[$value['param_id']] ?? '';
                if(! empty($name)){
                    $tmp = [
                        'id' => $value['param_id'],
                        'name' => $emp_map[$value['param_id']] ?? '',
                    ];
                    $employee[] = $tmp;
                }
            }elseif ($value['type'] == SeeRange::data_three){
                $name = $depart_map[$value['param_id']] ?? '';
                if(! empty($name)) {
                    $tmp = [
                        'id' => $value['param_id'],
                        'name' => $depart_map[$value['param_id']] ?? '',
                    ];
                    $depart2[] = $tmp;
                }
            }
        }

        return [$depart, $employee, $depart2];
    }

    //获取可见范围数据id
    public static function getRangeDataId($user,$data_type){
        $user_id = $user['id'];
        $depart_id = $user['depart_range'];
        $type = SeeRange::data_two;
        $type2 = [SeeRange::data_one,SeeRange::data_three];
        $type2 = implode(',',$type2);
        $depart_str = implode(',',$depart_id);
        $str = "(param_id = $user_id AND type = $type) OR (param_id IN({$depart_str}) AND type IN ({$type2}))";

        // 可见部门 可见人 可以看见
        $data_id = SeeRange::where('del_time',0)
            ->where('data_type', $data_type)
            ->where(function ($query) use($str) {
                $query->whereRaw($str);
            })->select('data_id')->get()->toArray();
        return array_unique(array_column($data_id,'data_id'));
    }

    //获取客户可见数据
    public static function customerRange($user,$search){
        // 销售人员/负责人 3协同人  可以看见
        $customer_id = CustomerInfo::where('del_time',0)
           ->where('data_id',$user['id'])
           ->whereIn('type',CustomerInfo::$see_man)
           ->select('customer_id')->get()->toArray();
        $return_id = array_unique(array_column($customer_id,'customer_id'));
        //可见范围id
        $rang_id = Self::getRangeDataId($user,SeeRange::type_one);
        //并和
        $return_id = array_unique(array_merge_recursive($return_id,$rang_id));

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('customer')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取施工单可见数据
    public static function constructionRange($user,$search){
        //单据中选择的签订负责协同人
        $construction_id = ConstructionInfo::where('del_time',0)
            ->where('employee_id',$user['id'])
            ->select('construction_id')
            ->get()->toArray();
        $return_id = array_unique(array_column($construction_id,'construction_id'));

        //可见范围id
        $return = Self::getRangeDataId($user,SeeRange::type_two);
        $return_id = array_unique(array_merge_recursive($return_id,$return));

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('construction')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取发货单可见数据
    public static function invoiceRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_three);

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('invoice_order')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取产品可见数据
    public static function productRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_four);

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('product')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取产品不可见数据
    public static function productRangeNot($user,$search){
        //不可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_four);

        //分社管理员
        if(empty($user['is_all_depart']) && ! empty($user['is_manager'])) {
            $depart = array_shift($user['rule_depart']);
            $depart_id = $depart['depart_id'] ?? 0;
            $id = DB::table('product')
                ->where('del_time',0)
                ->where('top_depart_id',$depart_id)
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取采购单可见数据
    public static function purchaseRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_five);

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('purchase_order')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取退换货单可见数据
    public static function returnExchangeOrderRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_six);
        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('return_exchange_order')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取合同可见数据
    public static function salesOrderRange($user,$search){
        //单据中选择的签订负责协同人
        $sales_order_id = SalesOrderInfo::where('del_time',0)
            ->whereIn('type',SalesOrderInfo::$man)
            ->where('data_id',$user['id'])
            ->select('sales_order_id')
            ->get()->toArray();
        $sales_order_id = array_unique(array_column($sales_order_id,'sales_order_id'));

        //指派后 可见范围id
        $return = Self::getRangeDataId($user,SeeRange::type_seven);

        $return_id = array_unique(array_merge_recursive($sales_order_id,$return));
        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('sales_order')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取供应商可见数据
    public static function supplierRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_nine);

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('supplier')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //获取活动包可见数据
    public static function sportsBagRange($user,$search){
        //可见范围id
        $return_id = Self::getRangeDataId($user,SeeRange::type_eight);

        if(! empty($search['top_depart_id']) && ! empty($user['is_all_depart'])){
            $id = DB::table('sports_bag')
                ->where('del_time',0)
                ->where('top_depart_id',$search['top_depart_id'])
                ->select('id')->get()->toArray();
            $id = array_column($id,'id');
            foreach ($return_id as $key => $value){
                if(! in_array($value,$id)) unset($return_id[$key]);
            }
        }

        return $return_id;
    }

    //产品不可见部门
    public static function productNotSeeRange($product_id){
        $return = [];
        $result = SeeRange::where('del_time',0)
            ->where('data_type', SeeRange::type_four)
            ->whereIn('data_id',$product_id)
            ->where('type',SeeRange::data_one)
            ->select('param_id as depart_id','data_id as product_id')
            ->get()->toArray();
        foreach ($result as $value){
            $return[$value['product_id']][] = $value['depart_id'];
        }

        return $return;
    }
}
