<?php

namespace App\Service;

use App\Model\BasicType;
use App\Model\Employee;
use App\Model\Product;
use App\Model\ProductActivity;
use App\Model\ProductActivityPrice;
use App\Model\Schedule;
use App\Model\ScheduleInfo;
use Illuminate\Support\Facades\DB;

class ScheduleService extends Service
{
    public function edit($data,$user){
        list($status,$msg) = $this->scheduleRule($data, $user, false);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = Schedule::where('id',$data['id'])->first();
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->save();

            $time = time();
            ScheduleInfo::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            if(! empty($data['day_time'])){
                $insert = [];
                foreach ($data['day_time'] as $value){
                    for ($i=0;$i<$value['num'];$i++){
                        foreach ($data['time_range'] as $range){
                            $date = date('Y-m-d',$range);
                            $start_time = strtotime($date . " " .$value['start_time']);
                            $end_time = strtotime($date . " " .$value['end_time']);
                            $insert[] = [
                                'schedule_id' => $model->id,
                                'crt_time' => $time,
                                'day' => $range ?? 0, //0点的时间戳
                                'start_time' => $start_time ?? 0, //加上开始时间段
                                'end_time' => $end_time ?? 0,//加上结束时间段
                            ];
                        }
                    }
                }
                ScheduleInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function add($data,$user){
        list($status,$msg) = $this->scheduleRule($data, $user);
        if(!$status) return [$status,$msg];

        try {
            DB::beginTransaction();

            $model = new Schedule();
            $model->start_time = $data['start_time'] ?? 0;
            $model->end_time = $data['end_time'] ?? 0;
            $model->mark = $data['mark'] ?? '';
            $model->crt_id = $user['id'];
            $model->depart_id = $data['depart_id'] ?? 0;
            $model->top_depart_id = $data['top_depart_id'] ?? 0;
            $model->save();

            $time = time();
            if(! empty($data['day_time'])){
                $insert = [];
                foreach ($data['day_time'] as $value){
                    for ($i=0;$i<$value['num'];$i++){
                        foreach ($data['time_range'] as $range){
                            $date = date('Y-m-d',$range);
                            $start_time = strtotime($date . " " .$value['start_time']);
                            $end_time = strtotime($date . " " .$value['end_time']);
                            $insert[] = [
                                'schedule_id' => $model->id,
                                'crt_time' => $time,
                                'day' => $range ?? 0, //0点的时间戳
                                'start_time' => $start_time ?? 0, //加上开始时间段
                                'end_time' => $end_time ?? 0,//加上结束时间段
                            ];
                        }
                    }
                }
                ScheduleInfo::insert($insert);
            }

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function del($data){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];

        try {
            DB::beginTransaction();

            $time = time();

            Schedule::where('id',$data['id'])->update(['del_time' => $time]);

            ScheduleInfo::where('schedule_id',$data['id'])
                ->where('del_time',0)
                ->update(['del_time' => $time]);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false,$exception->getMessage()];
        }

        return [true,''];
    }

    public function detail($data,$user){
        if($this->isEmpty($data,'id')) return [false,'请选择数据！'];
        $schedule = Schedule::where('del_time',0)
            ->where('id',$data['id'])
            ->first();
        if(empty($schedule)) return [false,'排班设置不存在或已被删除'];
        $schedule = $schedule->toArray();

        $schedule['crt_name'] = Employee::where('id',$schedule['crt_id'])->value('emp_name');
        $schedule['crt_time'] = $schedule['crt_time'] ? date("Y-m-d H:i:s",$schedule['crt_time']): '';

        return [true, $schedule];
    }

    public function getList($data,$user){
        $model = Schedule::TopClear($user,$data);
        $model = $model->where('del_time',0)
            ->select('id','crt_id','mark','start_time','end_time','crt_time')
            ->orderby('id', 'desc');

        if(! empty($data['mark'])) $model->where('mark', 'LIKE', '%'.$data['mark'].'%');
        if(! empty($data['crt_time'][0]) && ! empty($data['crt_time'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['crt_time']);
            $model->where('crt_time','>=',$return[0]);
            $model->where('crt_time','<=',$return[1]);
        }
        if(! empty($data['schedule'][0]) && ! empty($data['schedule'][1])) {
            $return = $this->changeDateToTimeStampAboutRange($data['schedule']);
            $model->where('start_time','>=',$return[0]);
            $model->where('end_time','<=',$return[1]);
        }

        $list = $this->limit($model,'',$data);
        $list = $this->fillData($list,$user);

        return [true, $list];
    }

    public function scheduleRule(&$data, $user, $is_add = true){
        if(empty($data['schedule'][0]) || empty($data['schedule'][1])) return [false,'排班日期不能为空'];
        $data['start_time'] = $this->changeDateToDate($data['schedule'][0]);
        $data['end_time'] = $this->changeDateToDate($data['schedule'][1]);
        $data['time_range'] = $this->getTimestampRange($data['start_time'],$data['end_time']);
        if(empty($data['day_time'])) return [false,'排班时间段不能为空'];
        foreach ($data['day_time'] as $value){
            if(empty($value['start_time'])) return [false,'排班时间段开始时间不能为空'];
            if(empty($value['end_time'])) return [false,'排班时间段结束时间不能为空'];
            if(empty($value['num'])) return [false,'排班车间数量不能为空'];
        }

        if($is_add){
            $model = Schedule::TopClear($user,$data);
            $bool = $model->where('del_time',0)
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->exists();
        }else{
            if(empty($data['id'])) return [false,'ID不能为空'];
            $model = Schedule::TopClear($user,$data);
            $bool = $model->where('del_time',0)
                ->where('id','<>',$data['id'])
                ->where('start_time', '<=', $data['end_time'])
                ->where('end_time', '>=', $data['start_time'])
                ->exists();
        }
        if($bool) return [false,'排班日期已经设置，请不要重复设置！'];

        //所属部门 以及 顶级部门
        if(empty($data['depart_id'])) $data['depart_id'] = $this->getDepart($user);
        $data['top_depart_id'] = $user['depart_map'][$data['depart_id']] ?? 0;

        return [true, $data];
    }

    public function fillData($data, $user){
        if(empty($data['data'])) return $data;

        $emp = Employee::whereIn('id',array_unique(array_column($data['data'],'crt_id')))
            ->pluck('emp_name','id')
            ->toArray();

        foreach ($data['data'] as $key => $value){
            $start_time = $value['start_time'] ? date("Y-m-d H:i",$value['start_time']) : '';
            $end_time = $value['end_time'] ? date("Y-m-d H:i",$value['end_time']) : '';
            $data['data'][$key]['schedule'] = $start_time . '——' . $end_time;
            $data['data'][$key]['crt_time'] = $value['crt_time'] ? date('Y-m-d H:i:s',$value['crt_time']) : '';
            $data['data'][$key]['crt_name'] = $emp[$value['crt_id']] ?? '';
        }

        return $data;
    }

    public function scheduleGetForConstruction($data,$user){
        if(empty($data['schedule'])) return [false,'请选择日期！'];
        $schedule = $this->changeDateToDate($data['schedule']);
        $model = Schedule::TopClear($user,$data);
        $schedule_id = $model->where('del_time',0)
            ->where('start_time', '<=', $schedule)
            ->where('end_time', '>=', $schedule)
            ->select('id')->get()->toArray();
        $schedule_id = array_column($schedule_id,'id');
        if(empty($schedule_id)) return [false,'该时间排班设置不存在或已被删除！'];

        $schedule_detail_list = ScheduleInfo::where('del_time',0)
            ->whereIn('schedule_id',$schedule_id)
            ->where('day',$schedule)
            ->where('is_use',ScheduleInfo::not_use)
            ->select('schedule_id','day','start_time','end_time')
            ->get()->toArray();
        if(empty($schedule_detail_list)) return [false,'该时间排班已满、不存在或已被删除！'];

        $return = [];
        foreach ($schedule_detail_list as $value){
            if(isset($return[$value['start_time'] . $value['end_time']])){
                $return[$value['start_time'] . $value['end_time']]['num'] += 1;
            }else{
                $tmp['schedule_id'] = $value['schedule_id'];
                $tmp['num'] = 1;
                $tmp['day_stamp'] = $value['day'];
                $tmp['day_start_stamp'] = $value['start_time'];
                $tmp['day_end_stamp'] = $value['end_time'];
                $tmp['day'] = date('Y-m-d',$value['day']);
                $tmp['start_time'] = date('H:i',$value['start_time']);
                $tmp['end_time'] = date('H:i',$value['end_time']);
                $return[$value['start_time'] . $value['end_time']] = $tmp;
            }
        }
        ksort($return);

        return [true, array_values($return)];
    }

    public function getTimestampRange($startTimestamp, $endTimestamp){
        // 将时间戳转换为日期格式
        $startDate = date('Y-m-d', $startTimestamp);
        $endDate = date('Y-m-d', $endTimestamp);

        // 创建日期范围
        $dateRange = [];
        $currentDate = strtotime($startDate);

        while ($currentDate <= strtotime($endDate)) {
            $dateRange[] = $currentDate;
            $currentDate = strtotime('+1 day', $currentDate);
        }

        // 将每个日期设置为当天的 0 点 0 分 0 秒并转换为时间戳
        $zeroTimestamps = [];

        foreach ($dateRange as $date) {
            $zeroTimestamps[] = strtotime(date('Y-m-d 00:00:00', $date));
        }

        return $zeroTimestamps;
    }
}
