<?php

namespace App\Service;

use App\Model\TableSetting;
use Illuminate\Support\Facades\DB;

class TableHeadService extends Service
{
    public function tableheadAdd($data, $user){
        if(empty($data['table_head'])) return [false,'自定义表头不能为空'];
        if(empty($data['menu_id'])) return [false, 'menu_id不能为空'];

        $time = time();
        $insert = [];
        foreach ($data['table_head'] as $value){
            if(empty($value['key'])) return [false, 'key不能为空'];
            if(empty($value['value'])) return [false, 'value不能为空'];
            if(empty($value['sort'])) return [false, 'sort不能为空'];
            if(! isset($value['is_show'])) return [false, 'is_show不能为空'];

            $insert[] = [
                'key' => $value['key'],
                'value' => $value['value'],
                'sort' => $value['sort'],
                'is_show' => $value['is_show'],
                'is_click_detail' => $value['is_click_detail'] ?? 0,
                'float' => $value['float'],
                'menu_id' => $data['menu_id'],
                'crt_time' => $time,
                'crt_id' => $user['id'],
            ];
        }

        DB::beginTransaction();
        try{
            TableSetting::where('del_time',0)
                ->where('crt_id',$user['id'])
                ->where('menu_id',$data['menu_id'])
                ->update(['del_time' => $time]);
            TableSetting::insert($insert);

            DB::commit();
        }catch (\Exception $exception){
            DB::rollBack();
            return [false, $exception->getMessage()];
        }

        return [true,''];
    }

    public function tableheadGet($data, $user){
        if(empty($data['menu_id'])) return [false,'menu_id不能为空！'];

        $header = TableSetting::where('del_time',0)
            ->where('menu_id',$data['menu_id'])
            ->where('crt_id',$user['id'])
            ->select('key','value','sort','is_show','is_click_detail','menu_id','float')
            ->get()->toArray();
        $header_map = array_column($header,null,'key');

        $header_default = config("header.{$data['menu_id']}") ?? [];
        foreach ($header_default as $key => $value){
            if(isset($header_map[$value['key']])) {
                //存在保存好的设置的表头 以下信息沿用
                $tmp = $header_map[$value['key']];
                $header_default[$key]['sort'] = $tmp['sort'];
                $header_default[$key]['is_show'] = $tmp['is_show'];
                $header_default[$key]['is_click_detail'] = $tmp['is_click_detail'];
                $header_default[$key]['float'] = $tmp['float'];
            }else{
                $header_default[$key]['sort'] = $key + 1;
                $header_default[$key]['is_show'] = 1;
                $header_default[$key]['is_click_detail'] = 0;
                $header_default[$key]['float'] = 0;
            }
        }

        return [true, $header_default];
    }
}
